/**	
*	@file	ChartData.java
*	@author	Jason Enderby
*	@date	06 Apr 2013
*	
*	@brief Stores and handles data describing a Chart
*	
*	Provides mutators and accessors to chart specific data,
*	abstracting away any complexities from handling 2 dimensional arraylists
*	
*/



import	java.util.ArrayList;

public class ChartData {
	protected ArrayList<ArrayList<String>>	m_Data;
	protected ArrayList<String>				m_Columns;
	protected final boolean					m_Error = true;
	
	/**
	*	Mutator for m_Data - internal data storage
	*
	*	@param data - data structure of the form (2 dimensional string arraylist)
	*
	*	@return - success
	*/
	private boolean setData(ArrayList<ArrayList<String>> data) {
		boolean success = false;
		if (data != null) {
			m_Data = data;
			success = true;
		}
		return success;
	}
	
	/**
	*	Accessor for m_Data - internal data storage
	*
	*	@return - data structure of the form (2 dimensional string arraylist)
	*/
	private ArrayList<ArrayList<String>> getData() {
		return m_Data;
	}
	
	/**
	*	Internal accessor for m_Columns - column data
	*
	*	@return - data structure of the form (string arraylist)
	*/
	private ArrayList<String> getColumns() {
		return m_Columns;
	}
	
	/**
	*	Accessor for m_Columns - column data
	*
	*	@param column - the column number to retreive data from
	*
	*	@return - title of the specified column; null for failure
	*/
	public String GetColumn(int column) {
		String title = null;
		if (column < getColumns().size()) {
			title = getColumns().get(column);
		}
		return title;
	}
	
	/**
	*	Mutator for m_Columns - column data
	*
	*	@param columns - string arraylist containing column names
	*
	*	@return - success
	*/
	public boolean SetColumns(ArrayList<String> columns) {
		boolean	success = false;
		if (columns != null) {
			m_Columns = columns;
			success = true;
		}
		return success;
	}
	
		
	/**
	*	Accessor for row column specific data
	*
	*	@param column - the column number to retreive data from
	*	@param row - the row number to retreive data from
	*
	*	@return - value stored at [row][column]; null for failure
	*/
	public String Get(int row, int column) {
		final int	CHECK_VALUE = 0;
		String	string = null;
		if (row < Height() && column < Width() 
			&& getData().get(CHECK_VALUE) != null) {
			string = getData().get(row).get(column);
		}
		return string;
	}
	
	/**
	*	Basic constructor
	*
	*	@return - new empty ChartData
	*/
	public ChartData() {
		setData(new ArrayList<ArrayList<String>>());
		SetColumns(new ArrayList<String>());
	}
	
	/**
	*	Constructor from 2 arrays 
	*
	*	@param titles - array of Strings storing column titles
	*	@param data - array of Strings storing row column specific data
	*
	*	@return - new ChartData storing specified data
	*/
	public ChartData(String[] titles, String[][] data) {
		setData(new ArrayList<ArrayList<String>>());
		SetColumns(new ArrayList<String>());
		for (int i = 0; i < titles.length; i++) {
			AddColumn(titles[i]);
		}
		for (int i = 0; i < titles.length; i++) {
			for (int j = 0; j < data[i].length; j++) {
				Add(i, data[i][j]);
			}
		}
	}
	
	/**
	*	Calculates height of data - number of rows
	*
	*	@return - height of data
	*/
	public int Height() {
		return getData().size();
	}
	
	/**
	*	Calculates width of data = number of columns
	*
	*	@return - width of data
	*/
	public int Width() {
		final int	CHECK_VALUE = 0;
		int			width = 0;
		if (getData().get(CHECK_VALUE) != null) {
			width = getData().get(CHECK_VALUE).size();
		}
		return width;
	}
	
	/**
	*	Adds a value to end of specified column
	*
	*	@param column - the column number to add data to
	*	@param value - the data to add
	*
	*	@return - success
	*/
	public boolean Add(int column, String value) {
		System.out.printf(m_Error ? "Adding value %s to %ds.\n" : "", 
			value, column);
		boolean	success = false;
		if (column <= Width()) {
			getData().get(column).add(value);
			success = true;
		}
		return success;
	}
	
	/**
	*	Adds a new column to the end of the column list
	*
	*	@param title - title of the new column to be added
	*
	*	@return - success
	*/
	public boolean AddColumn(String title) {
		System.out.printf(m_Error ? "Adding column: %s.\n" : "", title);
		boolean success = true;
		success &= getData().add(new ArrayList<String>());
		success &= getColumns().add(title);
		return success;
	}
	
	/**
	*	Converts data into a string to be printed to screen
	*
	*	@return - string representation of ChartData
	*/
	public String toString() {
		String string = "";
		for (int i = 0; i < Height(); i++) {
			string += GetColumn(i) + ">";
			for (int j = 0; j < Width(); j++) {
				string += Get(i, j) + "\t|";
			}
			string += "\n";
		}
		return string;
	}
	
	/**
	*	Validity check
	*
	*	@return - validity of data stored
	*/
	private boolean checkValidity() {
		boolean	valid = true;
		for (int i = 0; i < Height(); i++) {
			if (getData().get(i).size() != Width()) {
				valid = false;
			}
		}
		return valid;
	}
	
	/**
	*	Unit test
	*
	*	@param args - command line arguments; not used
	*
	*/
	public static void main(String[] args) {
		final String[][]	TEST_DATA = {{"5", "14", "3"}, 
			{"43", "1", "4"}, {"15", "28", "14"}, {"17", "18", "15"}};
		final String[]		TEST_COLUMNS = {"Test", "Data", "Yeah", "Hey"};
		ChartData	myData = new ChartData(TEST_COLUMNS, TEST_DATA);
		System.out.printf(myData.checkValidity()
			? "Valid%s" : "Invalid%s"," Construction.\n");
		System.out.printf("%s\n", myData);
	}
	
}