/**
 * 	@file ColourList.java
 * 	@author Andy Small
 * 	@version 1.0
 * 	@brief Handles an arraylist with colours generated of a specific colour scheme 
 * 	@date 21/3/2013
 *	@see ColourGenerator
 *
 *	This class allows ArrayLists of colours to be created, and passed to the 
 *	charts for colouring their visual data. 
 */


 
import java.util.ArrayList;
import java.awt.Color;

public class ColourList {
	private ArrayList<Color> m_ColourArrayList;
	
	/**
	* This method returns the array list of colours
	*
	* @return the ArrayList<Color> containing the colours generated
	*/
	private ArrayList<Color> GetList(){
		return m_ColourArrayList;
	}
	
	/**
	* This method sets the ArrayList, returning a boolean to check if the input
	* param is null or not
	*
	* @param arrayList is the ArrayList<Color> of colours that needs to be set
	*
	* @return the boolean after checking the validity of the input param
	*/
	private boolean SetList(ArrayList<Color> arrayList){
		boolean success = false;
		if (arrayList != null) {
					m_ColourArrayList = arrayList;
			success = true;
		}
		return success;
	}
		
	/**
	* This method returns a single colour, at an index calculated using the 
	* number of colours needed, and the length of the colour arraylist
	*
	* @param numberOfItems is an int equal to the number of colours required
	*
	* @return the colour at the specified index, calculated as described above
	*/	
	public Color Get(int numberOfItems) {
		int colourToReturn;
		if (GetList().size() > 0){
			colourToReturn = (numberOfItems%GetList().size());
		} else {
			colourToReturn = numberOfItems;
		}
		return GetList().get(colourToReturn);
	}		

	/**
	* This method adds a single colour onto the end of the ArrayList
	*
	* @param colour is the generated Color to be added to the ArrayList
	*
	* @return the boolean, corresponding to if the input colour is valid
	*/
	public boolean Add(Color colour) {
		boolean success = false;
		if (colour != null) {
			GetList().add(colour);
			success = true;
		}
		return success;
	}
	
	/**
	* This method is one of the two constructors, this one is used if a 
	* non-user defined colour scheme is needed
	*
	* @param numberOfItems is the number of colours needed
	* @param colourScheme is the name of the colourScheme chosen
	*
	*/
	public ColourList(int numberOfItems, String colourScheme) {
		SetList(new ArrayList<Color>());
		ColourGenerator gen = new ColourGenerator();
		for (int i = 0; i < numberOfItems; i++){
			Add(gen.GetColour(colourScheme));
		}
	}

	/**
	* This method is the second constructor, used if a user defined colour 
	* scheme is being used
	*
	* @param userColourList is the ArrayList<Color> created by the user
	*
	*/
	public ColourList(ArrayList<Color> userColourList) {		
		SetList(new ArrayList<Color>(userColourList));
	}

	/**
	* This method produces a string representation of the ArrayList<Color>
	*
	* @return the String representation of the ArrayList<Color>
	*/
	public String toString(){
		String string = "";
		for (int i = 0; i < GetList().size(); i++){
			string += GetList().get(i) + "\n";
		}
		return string;
	}	
	
	/**
	* This method is the main method, and is never called by the software, 
	* it is only here to test the ArrayList<Color> with test values
	*
	* @param args is the String array equal to each term entered after the
	* program call in the command line
	*
	*/
	public static void main(String args[]){
		final int TEST_ITEMS = 10;
		ColourList testList = new ColourList(TEST_ITEMS,ColourGenerator.WARM);
		System.out.println(testList);
	}
}