/**
 * 	@file PieChart3D.java
 * 	@author Andy Small
 * 	@version 1.0
 *	@brief Stores a user defined PieChart3D to be displayed on screen
 * 	@date 03 Apr 2013
 *	@see Chart, ColourGenerator, ColourList, ChartData
 *
 *	Provides an interface for designing and displaying a PieChart3D
 *	Storing internal chart data and providing visual functions.
 * 
 */
 


import	org.jfree.chart.ChartFactory;
import	org.jfree.chart.JFreeChart;
import	org.jfree.chart.plot.PiePlot;
import	org.jfree.data.general.DefaultPieDataset;
import	org.jfree.data.general.PieDataset;
import	org.jfree.util.Rotation;
import  java.util.ArrayList;
import  java.awt.Color;
import  org.jfree.data.general.Dataset;

import	javax.swing.JFrame;



public class PieChart3D extends Chart {
	
	/**
	* Construct a PieChart3D
	*
	* @param data is the data that shall be used to create the chart
	* @param title is the title to be used upon creation of the chart
	* @param colourScheme is a string of the type of colour to use 
	* @param userList is the ArrayList<Color> of user defined colours
	* 
	*/
	public PieChart3D(ChartData data, String title, String colourScheme, 
													ArrayList<Color> userList){
		super(data, title, colourScheme, userList);
		SetLegend(GetChart().getLegend());
		SetDate(true);
	}
	
	/**
	* Create a chart of type PieChart3D. 
	*
	* @param plotData is the data that shall be used to create the chart
	* @param title is the title to be used upon creation of the chart
	*
	* @return the newly created PieChart3D as chart 
	*/
	JFreeChart CreateChart(Dataset plotData, String title) {
		final long	START_ANGLE = 180;
		
		JFreeChart chart = ChartFactory.createPieChart3D(title,	// Chart title
			(PieDataset) plotData,	//	Data
			true,					//	Include legend
			true,					//	Include tooltips
			false);					//	Do not include URLs

		PiePlot plot = (PiePlot) chart.getPlot();
		plot.setStartAngle(START_ANGLE);
		plot.setDirection(Rotation.CLOCKWISE);
		return chart;
	}
	
	/**
	* Change the currently selected chart to have a colourScheme 
	* as selected by the user 
	*
	* @param colourScheme is the Colour Scheme which has been selected by the user
	* which is to be mapped onto the charts data.
	*
	* @return boolean indicating True if colourScheme was successfully implemented
	* or false otherwise.
	*/
	boolean ApplyColour(ColourList colourScheme) {
		boolean	success = false;
		PiePlot	plot = (PiePlot) GetChart().getPlot();
		if (plot != null) {
			for (int i = 0; i < GetData().Width(); i++) {
				plot.setSectionPaint(String.format("%s:%s", 
				GetData().GetColumn(0),
				Double.parseDouble(GetData().Get(0, i))),
					GetColour().Get(i));
			}
			success = true;
		}
		return success;
	}
	
	/**
	* Create a dataset of type DefaultPieDataset using data of type ChartData 
	*
	* @param data is the data that shall be used to create the chart
	*
	* @return the newly created dataset as result 
	*/
	public Dataset CreateDataset(ChartData data) {
		DefaultPieDataset result = new DefaultPieDataset();
		for (int i = 0; i < data.Width(); i++) {
			result.setValue(String.format("%s:%s", data.GetColumn(0),
										Double.parseDouble(data.Get(0, i))),
										Double.parseDouble(data.Get(0, i)));
		}
		return result;
	}
	
	/**
	*	Unit test
	*	@param args - command line arguments (not used)
	*/
	public static void main(String[] args) {
		final int			WINDOW_SIZE = 500;
		final String[][]	TEST_DATA = {{"5","14","3","6","43","1","4","12"}};
		final String[]		TEST_COLUMNS = {"Test"};
		JFrame		frame = new JFrame("PieChart3D Test");
		ChartData 	data = new ChartData(TEST_COLUMNS, TEST_DATA);
		PieChart3D	chart = new PieChart3D(data, 
									"3D Pie Chart Test", ColourGenerator.RANDOM,null);
		System.out.println(chart);
		frame.add(chart);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setSize(WINDOW_SIZE, WINDOW_SIZE);
		frame.setVisible(true);		
	}
} 