/**
* 	@file StackedBarChart3D.java
* 	@author Connor Lynch
* 	@date 03 Apr 2013
* 
*	@see Chart, ColourGenerator, ColourList, ChartData
*
*	@brief Stores a user defined StackedBarChart3D to be displayed on screen
*	
*	Provides an interface for designing and displaying a StackedBarChart3D
*	Storing internal chart data and providing visual functions.
* 
*/



import	org.jfree.chart.ChartFactory;
import	org.jfree.chart.JFreeChart;
import  org.jfree.chart.plot.CategoryPlot;
import  org.jfree.chart.plot.PlotOrientation;
import  org.jfree.data.category.CategoryDataset;
import  org.jfree.data.category.DefaultCategoryDataset;
import  org.jfree.data.general.Dataset;
import java.util.ArrayList;
import java.awt.Color;
import	org.jfree.chart.renderer.category.StackedBarRenderer3D;

import  javax.swing.JFrame;


public class StackedBarChart3D extends Chart {
	

	/**
	* Construct a StackedBarChart3D
	*
	* @param data is the data that shall be used to create the chart
	* @param title is the title to be used upon creation of the chart
	* @param colourScheme is a string of the type of colour to use 
	* @param userList is the ArrayList<Color> of user defined colours
	* 
	*/
	public StackedBarChart3D(ChartData data, String title, String colourScheme, 
													ArrayList<Color> userList){
		super(data, title, colourScheme, userList);
		SetLegend(GetChart().getLegend());
		SetDate(true);
	}


	/**
	* Create a chart of type StackedBarChart3D. 
	*
	* @param plotData is the data that shall be used to create the chart
	* @param title is the title to be used upon creation of the chart
	*
	* @return the newly created StackedBarChart3D as chart 
	*/
	public JFreeChart CreateChart(Dataset plotData, String title) {
	    JFreeChart chart = ChartFactory.createStackedBarChart3D(
		title,						//	Chart title
		null,					// 	Domain/x axis label
		null, 						// 	Range/y axis label 
		(CategoryDataset) plotData,	//	Data
		PlotOrientation.VERTICAL,	//	Orientation
		true,				//	Include legend
		true,				//	Include tooltips
		false);				//	Do not include URLs
		
		return chart;
	}
	
	
	/**
	* Change the currently selected chart to have a colourScheme 
	* as selected by the user 
	*
	* @param colourScheme is the Colour Scheme which has been selected by the user
	* which is to be mapped onto the charts data.
	*
	* @return boolean indicating True if colourScheme was successfully implemented
	* or false otherwise.
	*/
	boolean ApplyColour(ColourList colourScheme) {
		boolean	success = false;
		CategoryPlot plot = (CategoryPlot) GetChart().getPlot();
		StackedBarRenderer3D renderer = (StackedBarRenderer3D)plot.getRenderer();
		if (plot != null) {
			for (int row = 0; row < GetData().Width(); row++) {
					renderer.setSeriesPaint(row, GetColour().Get(row));
			}
			success = true;
		}
		return success;
	}

	/**
	* Create a dataset of type DefaultCategoryDataset using data of type ChartData 
	*
	* @param data is the data that shall be used to create the chart
	*
	* @return the newly created dataset as result 
	*/
	Dataset CreateDataset(ChartData data) {
		DefaultCategoryDataset result = new DefaultCategoryDataset();
		for (int col = 0; col < data.Height(); col++) {
			for (int row = 0; row < data.Width(); row++) {
				result.addValue(Double.parseDouble(data.Get(col, row)),
				"Row " + (row+1), data.GetColumn(col) );
			} 
		}
	    return result;
	}

	/**
	*	Unit test
	*	@param args - command line arguments (not used)
	*/
	public static void main(String[] args) {
		final int			WINDOW_SIZE = 700;
		final String[][]	TEST_DATA = {{"5","14","3","25","32",
										"23","21","6","5","11"},
										{"23","16","12","42","2",
										"2","12","4","25","19"},
										{"45","34","13","15","22",
										"5","11","32","12","43"}};
		final String[]		TEST_COLUMNS = {"Test","Data","Yeah"};
		JFrame		frame = new JFrame("StackedBarChart3D Test");
		ChartData 	data = new ChartData(TEST_COLUMNS, TEST_DATA);
		StackedBarChart3D	chart = new StackedBarChart3D(data, "Stacked Bar Test", 
													ColourGenerator.WARM,null);
		System.out.println(chart);
		frame.add(chart);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setSize(WINDOW_SIZE, WINDOW_SIZE);
		frame.setVisible(true);		
	}
	
	
} 